function ccd = get_hierarchical_cluster_coefficient_ccd(connectivity_matrix, hierarchy_level)
    % ccd = get_hierarchical_cluster_coefficient_ccd(connectivity_matrix, hierarchy_level)
    % 
    %  input:
    %           connectivity_matrix = Adjacency matrix
    %           hierarchy_level     = Level of hierarchy of cluster coefficient
    % output:
    %           ccd                 = List of hierarchical cluster coefficients
    %
    % Function calculates hierarchical cluster doefficient of desired hierarchy
    % level for all nodes. (Note: For hierarchy level = 1 the hierarchical 
    % clustering coefficient is identical to the "classic" clustering coefficient.
    
    % Christoph Feenders (ne Echtermeyer), 2010-2013

    no_nodes        = length(connectivity_matrix);
    ccd             = NaN(1, no_nodes);
    max_path_length = hierarchy_level;
    distance_matrix = get_shortest_path_matrix( connectivity_matrix, max_path_length );
    distance_matrix = set_diagonal_to_zero(distance_matrix);
    distance_d      = hierarchy_level;
    
    for i = 1:no_nodes
        ccd(i) = get_hierarchical_cluster_coefficient_ccdi( distance_matrix, distance_d, i );
    end
end


function ccdi = get_hierarchical_cluster_coefficient_ccdi( distance_matrix, distance_d, node_i)
    ndi = get_no_of_ring_elements_ndi(distance_matrix, distance_d, node_i);
    if( ndi > 1 )
        edi = get_no_of_links_in_ring_subnetwork_edi(distance_matrix, distance_d, node_i);
        ccdi = edi / (ndi * (ndi - 1));
    else
        ccdi = 0;
    end
end


function matrix = set_diagonal_to_zero( input_matrix ) 
    no_nodes = length(input_matrix);
    matrix   = input_matrix;
    matrix(eye(no_nodes)==1) = 0;
end
    
    
function ring_elements_Rdi = get_ring_elements_Rdi(distance_matrix, distance_d, node_i)    
    p1 = find( distance_matrix(node_i, :) == distance_d );
    p2 = find( distance_matrix(:, node_i) == distance_d );
    ring_elements_Rdi = unique([p1, p2']);
end


function no_of_ring_elements_ndi = get_no_of_ring_elements_ndi(distance_matrix, distance_d, node_i)    
    no_of_ring_elements_ndi = length( get_ring_elements_Rdi(distance_matrix, distance_d, node_i) );
end


function A_gamma = get_ring_subnetwork_gammadi(distance_matrix, distance_d, node_i)
    ring_elements_Rdi = get_ring_elements_Rdi(distance_matrix, distance_d, node_i);
    A_gamma = (distance_matrix(ring_elements_Rdi, ring_elements_Rdi) == 1);
end


function no_of_links_in_subnet_edi = get_no_of_links_in_ring_subnetwork_edi(distance_matrix, distance_d, node_i)
    A_gamma = get_ring_subnetwork_gammadi(distance_matrix, distance_d, node_i);
    no_of_links_in_subnet_edi = sum(sum(A_gamma));
end
