% Calculates the length of the shortest path between any pair of nodes in
% a network.
%
% Syntax:
%           shortest_path_mtx = get_shortest_path_matrix(connectivity_mtx) 
%           shortest_path_mtx = get_shortest_path_matrix(connectivity_mtx, ...
%                                                               max_path_length) 
% 
%   where
%           connectivity_mtx  = adjacency matrix
%           max_path_length   = maximal path-length to consider
%           shortest_path_mtx = matrix of minimum path lengths
%
% Function calculates minimum path-length between any pair of nodes. Paths
% of length greater than the (optional) maximal path-length are not 
% considered. Nodes that are not connected by a path up to the maximal 
% path-length are are assigned a path length of zero, i.e.
%
%    shortest_path_mtx(i,j) = length of shortest path from i to j
%                             or 0 if path does not existant or 
%                             longer than max_path_length
%
%   Copyright Christoph Feenders, 2009-2012

%% part of /home/christoph/synced/programming/matlab/hard link targets

function shortest_path_mtx = get_shortest_path_matrix(connectivity_mtx, ...
                                                                  varargin) 
 
    assert(size(connectivity_mtx, 1) == size(connectivity_mtx, 2), ...
                            'Connectivity matrix must be a square matrix.')

    no_nodes          = size(connectivity_mtx, 1);
    connectivity_mtx  = connectivity_mtx * 1.0;
    paths_of_length_k = connectivity_mtx;
    shortest_path_mtx = connectivity_mtx;
 
    switch nargin
        case 1
            max_path_length = no_nodes-1;  % longest path possible
        case 2
            max_path_length = min(no_nodes-1, varargin{1});
    end
    
    for k = 2:max_path_length

        if all(shortest_path_mtx(:) > 0)
            % shortest path for all nodes already found
            return
        end
            
        paths_of_length_k = paths_of_length_k * connectivity_mtx;
        no_path_boolidx   = (shortest_path_mtx == 0);
        shortest_path_mtx(no_path_boolidx) = ...
                                  k * (paths_of_length_k(no_path_boolidx) > 0);

    end
end
