function [centroids, noOfpointsInCluster, assignments] = S5h_Kmeans(points_to_cluster, no_of_clusters_k, initial_centroids)
    % [centroids, noOfpointsInCluster, assignments] = S5h_Kmeans(points_to_cluster, no_of_clusters_k, initial_centroids) 
    % 
    %  input:
    %           points_to_cluster   = Data points to cluster
    %           no_of_clusters_k    = Number of clusters
    %           initial_centroids   = optional initial centroids
    % output:
    %           centroids           = Identified cluster centroids
    %           noOfpointsInCluster = Number of points assigned to cluster
    %           assignments         = IDs of clusters nodes have been assigned to
    %
    % "Beyond the Average"-workflow -- Step 5
    % Helper function k-means to cluster points to k motif groups.
    
    % Christoph Echtermeyer, 2009-2010

    % For a nice illustration of how the algorithm works see:
    % http://en.wikipedia.org/wiki/K-means_clustering#Standard_algorithm

    [no_of_data_points, data_dim] = size(points_to_cluster);
    data_min  = min(points_to_cluster); % re-used later
    data_diff = max(points_to_cluster) - data_min ;
    if nargin < 3 % if no initial centroids given, initialise them randomly
        centroids = (rand(no_of_clusters_k, data_dim) ...
                        .* repmat(data_diff, no_of_clusters_k, 1)) ...
                        + repmat(data_min, no_of_clusters_k, 1);
    else
        centroids = initial_centroids;
    end

    if no_of_data_points < no_of_clusters_k
        error(sprintf('Can not create %i clusters for $i data-points.', no_of_clusters_k, no_of_data_points))
    end
    
    assignments         = zeros(no_of_data_points, 1);
    noOfpointsInCluster = zeros(no_of_clusters_k, 1);
    centroid_change     = 1;
    while centroid_change > 0. % repeat until centroids stay fixed
        % assign each datapoint to the closest centroid
        for d = 1:no_of_data_points
            node_assignment = 1; % default assignment
            smallest_dist = Inf;
            for c = 1:no_of_clusters_k;
                diff = points_to_cluster(d,:) - centroids(c,:);
                dist = dot(diff, diff);
                if dist < smallest_dist
                    node_assignment = c;
                    smallest_dist = dist;
                end
            end
            assignments(d) = node_assignment;
        end
        % recalculate the positions of the centroids
        centroids_previous = centroids;
        centroids = zeros(no_of_clusters_k, data_dim);
        for c = 1:no_of_clusters_k 
            in_cluster_idx = find(assignments==c);
            noOfpointsInCluster(c) = sum(in_cluster_idx > 0);
            if noOfpointsInCluster(c) > 0
                if noOfpointsInCluster(c) > 1
                    centroids(c,:) = sum(points_to_cluster(in_cluster_idx,:)) / noOfpointsInCluster(c);
                else % the one cluster-member is the centroid itself
                    centroids(c,:) = points_to_cluster(in_cluster_idx,:);
                end
            else % reassign centroid randomly
                centroids(c,:) = (rand(1, data_dim) .* data_diff) + data_min;
            end
        end
        % determine degree to which centroids have changed
        centroid_change = sum(sum( (centroids - centroids_previous).^2 ) );
    end
end
