function centroids = S5h_centroidFinder(data, nbCluster)
    % centroids = S5h_centroidFinder(data, nbCluster) 
    % 
    %  input:
    %           data      = Data points to cluster
    %           nbCluster = Number of clusters
    % output:
    %           centroids = Suggested cluster centroids
    %
    % "Beyond the Average"-workflow -- Step 5
    % Helper function to determine good cluster centroids (according to 
    % k-means++ algorithm).
    
    % Christoph Echtermeyer, 2009


% k-means++ algorithm
%   1. Choose one centroid uniformly at random from among the data points.
%   2. For each data point x, compute D(x), the distance between x and the nearest centroid that has already been chosen.
%   3. Add one new data point at random as the new centroid that follows xk+1 > D(xk)2.
%   4. Repeat Steps 2 and 3 until k centers have been chosen.
%   5. Now that the initial centers have been chosen, proceed using standard k-means clustering.

    [no_of_data_points, dim_data] = size(data);
    centroids = zeros(nbCluster, dim_data);

    %   1. Choose one centroid uniformly at random from among the data points.
    if(octave_running())
        ri = unidrnd(no_of_data_points);
    else
        ri = randi(no_of_data_points);
    end
    centroids(1, :) = data(ri, :);

    %   4. Repeat Steps 2 and 3 until k centers have been chosen.
    for k = 2:nbCluster
        %   2. For each data point x, compute D(x), the distance between x and the nearest centroid that has already been chosen.
        distances = zeros(no_of_data_points, 1) + inf; % node's dist to closest cluster
        for i = 1:no_of_data_points
            for j = 1:(k-1)
                distances(i) = min( distances(i), sum( (centroids(j, :) - data(i, :)).^2 ) );
            end
        end
        
        %   3. Add one new data point at random as the new centroid that follows xk+1 > D(xk)2.
        new_centroid_index = myrandsample(1:no_of_data_points, distances);
        centroids(k, :) = data(new_centroid_index, :);
    end
end


% using sub-function rather than private/nested function for octave-compatibility
function sample = myrandsample(potential_samples, probability_weights)
% note: probability_weights do not need to sum to unity but must be >= 0 only

    if(sum(probability_weights) > 0)

        P_X_leq_x = 0;
        rn = rand; % open interval (0,1)
        probabilities = probability_weights / sum(probability_weights);
        for i = 1:length(potential_samples)
            if( (P_X_leq_x <= rn) && (rn < P_X_leq_x+probabilities(i)) )
                sample = potential_samples(i);
                return;
            else
                P_X_leq_x = P_X_leq_x + probabilities(i);
            end
        end
    end
    % When arriving here all nodes lie directly on a centroid,
    % i.e. at least 2 nodes are on top of each other.
    % In this case return an arbitrary sample.
    sample = potential_samples(1);
end