function workflow_plots(handles)
    % workflow_plots(handles)
    % 
    %  input:
    %           handles = Handle structure of BtA-GUI
    % output:
    %           (none)
    %
    % "Beyond the Average"-workflow
    % Helper function to plot results.
    
    % Christoph Feenders (ne Echtermeyer), 2009-2013
        
    eigenspace_projection = handles.eigenspace_projection;
    bandwidth             = handles.bandwidth;
    probabilities         = handles.probabilities;
    w                     = handles.w;
    k                     = handles.k;
    assignments           = handles.assignments;
    singular_nodes        = handles.singular_nodes;
    regular_nodes         = handles.regular_nodes;
    t_x                   = handles.t_x;
    t_y                   = handles.t_y;
    density_matrix        = handles.density_matrix; 

    % set appropriate symbol-size
    if(octave_running())
        symbol_size = 5;
    else % assume it is matlab
        symbol_size = 50;
    end

    % plots
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    axes(handles.axes1);
    if(octave_running())
        scatter(eigenspace_projection(:,1), eigenspace_projection(:,2), symbol_size, probabilities);
    else % assume it is matlab
        scatter(eigenspace_projection(:,1), eigenspace_projection(:,2), symbol_size, probabilities, ...
            '^', 'filled');
    end
    xlabel('PC1');
    ylabel('PC2');
    if isfield(handles, 'explained_variance') > 0
        title( sprintf('Principal components represent %.1f%% of variance', 100*handles.explained_variance) );
    else
        title('Principal components');
    end
    if isfield(handles, 'PC_min_x') > 0
        xlim([handles.PC_min_x, handles.PC_max_x])
        ylim([handles.PC_min_y, handles.PC_max_y])
    else
        axis('tight');
    end
    minimal_extent = min(max(eigenspace_projection) - min(eigenspace_projection));
    if( minimal_extent > 0 ) % otherwise plotting error!
        colorbar('WestOutside');
    end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    axes(handles.axes2);
    no_of_nodes = size(eigenspace_projection, 1);
    m = mean(probabilities);
    sdev = std(probabilities);
    [sorted, i] = sort(probabilities);

    diffs = sorted(2:end) - sorted(1:end-1);
    diffs_x = (1:size(diffs, 1)) + 0.5;
    scaling_factor = 0.5 * max(sorted)/max(diffs);
    stem(diffs_x, scaling_factor*diffs, 'fill', 'c');
    hold on;
    plot(1:w, sorted(1:w), 'r+', w+1:no_of_nodes, sorted(w+1:no_of_nodes), ...
        'g+', 'LineWidth', 2);
    plot([1, no_of_nodes], [m, m], 'k', [1, no_of_nodes], [m+sdev, m+sdev], ...
        'b', [1, no_of_nodes], [m-sdev, m-sdev], 'b');
    hold off;
    axis('tight');
    xlabel('node');
    ylabel('~ probability');
    title( sprintf('Sorted node probabilities (w = %i)', w) );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    axes(handles.axes3);
    contourf(t_x, t_y, density_matrix)
    hold on;
    plot(singular_nodes(:,1), singular_nodes(:,2), 'm^', 'markersize', 5, ...
        'MarkerFaceColor', 'm');
    plot(regular_nodes(:,1),  regular_nodes(:,2),  'bo', 'markersize', 5);
    hold off;
    xlabel('PC1');
    ylabel('PC2');
    title( sprintf('Parzen-window KDE (bandwidth = %.2f)', bandwidth) );
    if isfield(handles, 'PC_min_x') > 0
        xlim([handles.PC_min_x, handles.PC_max_x])
        ylim([handles.PC_min_y, handles.PC_max_y])
    else
        axis('tight');
    end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    axes(handles.axes4);
    SD = std(eigenspace_projection);
    plot(regular_nodes(:,1)/SD(1), regular_nodes(:,2)/SD(2), 'ko');
    hold on;
    if(octave_running())
        scatter(singular_nodes(:,1)/SD(1), singular_nodes(:,2)/SD(2), symbol_size, assignments );
    else % assume it's matlab
        scatter(singular_nodes(:,1)/SD(1), singular_nodes(:,2)/SD(2), symbol_size, assignments, ...
            '^', 'filled');
    end
    hold off;
    xlabel('PC1');
    ylabel('PC2');
    title( sprintf('motif grouping (k = %i)', k) );
    if isfield(handles, 'PC_min_x') > 0
        xlim([handles.PC_min_x/SD(1), handles.PC_max_x/SD(2)])
        ylim([handles.PC_min_y/SD(1), handles.PC_max_y/SD(2)])
    else
        axis('tight');
    end

end
