function workflow(filename)
    % workflow(filename)
    % 
    %  input:
    %           filename = Name of csv-file that contains the connectivity-
    %                      matrix to analyse.
    % output:
    %           (none) 
    %
    % The file is read in, the "Beyond the Average"-workflow is executed,
    % and output graphics are saved in pdf-format.
    
    % Christoph Echtermeyer, 2009-2010

    % --------------------------------------------------------------------------
    %    Parzen-window bandwidth
    %        list of bandwidths (bw) to use (0 < bw <= 1)
    %        automatic settings will be used if list empty
    % --------------------------------------------------------------------------
    bandwidths = []
    % toggles adaptation of kernel shape independently for each PC-axis
    handles.adapt_kernel = true;

    % --------------------------------------------------------------------------
    %    Selection of singular nodes
    %        list of number of singular nodes (w) to use (1 <= w < number of nodes)
    %        automatic settings will be used if list empty
    % --------------------------------------------------------------------------
    ws = []

    % --------------------------------------------------------------------------
    %    k-means grouping of singular nodes
    %        list of number of motif groups (k) to use (1 <= k <= w)
    %        automatic settings will be used if list empty
    % --------------------------------------------------------------------------
    ks = []
    % alternatively to k-means the overlap-cliques can be determined
%    handles.motif_grouping_function = 'kmeans';
    handles.motif_grouping_function = 'overlap';

    % not using matlab-package structure to ensure octave-compatibility
    % adding paths instead
    base_dir = pwd;
    addpath( sprintf('%s/%s', base_dir, '!workflow') );
    addpath( sprintf('%s/%s', base_dir, '!graphStats') );
    addpath( sprintf('%s/%s', base_dir, '!dataHandling') );

    if(octave_running())
        % load equivalent to matlab 'ksdensity' function
        pkg load econometrics
        pkg load statistics
    end

    % set name for output
    r = strfind(filename, '.csv');
    handles.file_basename = filename(1 : r(end)-1);
    [handles.connectivity_matrix, handles.no_of_nodes] = get_matrix_from_csv_file(filename);

    % remove isolated nodes from network
    [handles.connectivity_matrix, handles.no_of_nodes] = remove_isolated_nodes(handles.connectivity_matrix);

    % fundamental analysis for subsequent analyses
    % get local statistics
    [handles.statistics, handles.stats_description] = S1_calculate_stats(handles.connectivity_matrix, handles);
    % perform principal comonent analysis
    pcs = 2; % number of principal components
    [handles.eigenspace_projection, handles.explained_variance] = S2_PCA(handles.statistics, pcs);

    [handles.t_x, handles.t_y, handles.grid_points] = S0h_get_grid_points(handles);

    % create axes-objects for plots
    handles.fig1 = figure('Visible', 'off'); handles.axes1 = axes();
    handles.fig2 = figure('Visible', 'off'); handles.axes2 = axes();
    handles.fig3 = figure('Visible', 'off'); handles.axes3 = axes();
    handles.fig4 = figure('Visible', 'off'); handles.axes4 = axes();
    handles.fig5 = figure('Visible', 'off'); handles.axes5 = axes();

    % evoking workflow-series
    % eventually set defaults
    if (length(bandwidths) == 0)
        bandwidth_list = [1]; 
    else
        bandwidth_list = bandwidths;
    end;

    for bandwidth = bandwidth_list
        handles.bandwidth = bandwidth;

        % estimate probabilites for all nodes
        handles.probabilities = S3_kernel_smoothing(handles.eigenspace_projection, ...
            handles.eigenspace_projection, handles.bandwidth, handles.adapt_kernel);

        % plot preparation
        density = S3_kernel_smoothing(handles.grid_points, ...
            handles.eigenspace_projection, handles.bandwidth, handles.adapt_kernel);
        handles.density_matrix = reshape(density, 100, 100);

        % eventually set defaults
        if (length(ws) == 0) 
            w_list = S4_determine_w(handles.probabilities); 
        else
            w_list = ws;
        end

        % select new w
        for w = w_list 
            handles.w = w;
        
            % set singular_nodes according to iterator variable w
            [sorted, i] = sort(handles.probabilities);
            handles.sorted_index = i;
            handles.singular_nodes = handles.eigenspace_projection( i(1:handles.w), : );
            handles.regular_nodes  = handles.eigenspace_projection( i(handles.w+1:handles.no_of_nodes), : );

            % eventually set defaults
            if (length(ks) == 0)
                k_list = S5f_determine_k(handles); 
            else
                k_list = ks;
            end
            
            for k = k_list
                handles.k = k;
            
                % group singular nodes
                SD = std(handles.eigenspace_projection);
                normalised_points = handles.singular_nodes ./ (ones(size(handles.singular_nodes,1), 1) * SD);
                % constant rows have SD=0, so normalised data contains NaNs; replace
                % them with zeros
                normalised_points(isnan(normalised_points)) = 0;

                switch handles.motif_grouping_function
                    case 'kmeans'
                        [handles.centroids, handles.noOfpointsInCluster, handles.assignments] = S5b_group_nodes(normalised_points, handles.k);
                    case 'overlap'
                        [dummy_k, handles.centroids, handles.noOfpointsInCluster, handles.assignments] = S5f_fast_group_nodes(normalised_points);
                end
                    
                workflow_plots(handles);

                % make network fingerprint
                workflow_fingerprint(handles)
  
                save_plot(handles);
                save_analysis_results(handles);
                
            end
        end
    end
end
